diff --git a/src/backend/replication/logical/launcher.c b/src/backend/replication/logical/launcher.c
index 27c3a91fb7..2ca9fb22e8 100644
--- a/src/backend/replication/logical/launcher.c
+++ b/src/backend/replication/logical/launcher.c
@@ -1149,6 +1149,7 @@ ApplyLauncherMain(Datum main_arg)
 		MemoryContext subctx;
 		MemoryContext oldctx;
 		long		wait_time = DEFAULT_NAPTIME_PER_CYCLE;
+		bool		launched_something = false;
 
 		CHECK_FOR_INTERRUPTS();
 
@@ -1201,6 +1202,8 @@ ApplyLauncherMain(Datum main_arg)
 										 sub->dbid, sub->oid, sub->name,
 										 sub->owner, InvalidOid,
 										 DSM_HANDLE_INVALID);
+				/* remember that we launched something; see comment below */
+				launched_something = true;
 			}
 			else
 			{
@@ -1214,6 +1217,17 @@ ApplyLauncherMain(Datum main_arg)
 		/* Clean the temporary memory. */
 		MemoryContextDelete(subctx);
 
+		/*
+		 * Launching a worker uses MyLatch to wait for the worker to attach,
+		 * resetting it. We use the same latch to be signalled about
+		 * subscription changes and woerkers exiting, so we might have missed
+		 * some notifications, if those events happened concurrently. To work
+		 * around that, do another iteration immediately, if we launched
+		 * anything in this iteration.
+		 */
+		if (launched_something)
+			continue;
+
 		/* Wait for more work. */
 		rc = WaitLatch(MyLatch,
 					   WL_LATCH_SET | WL_TIMEOUT | WL_EXIT_ON_PM_DEATH,
